
# q-expansion of newform 1734.4.a.h, downloaded from the LMFDB on 30 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1734
weight = 4
poly_data = [-15, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 4], 1]]

hecke_ring_character_values = None
aps_data = [[-2, 0], [-3, 0], [-6, 1], [-8, -1], [12, -2], [38, -2], [0, 0], [116, 2], [0, -11], [-30, 5], [-80, -3], [178, -11], [294, 12], [-100, -12], [-96, -26], [-42, 26], [-180, 14], [106, -31], [68, -22], [384, -33], [-26, 48], [112, 45], [180, 34], [-294, 20], [-194, -58], [198, 34], [-928, -50], [108, -86], [586, 15], [126, -10], [1544, 48], [468, -10], [1194, -64], [2476, 14], [-2394, 12], [368, -64], [806, 12], [1300, -12], [-2448, 77], [-798, 133], [-108, -136], [-3278, -35], [3456, -6], [-962, 118], [-918, 71], [4120, 63], [916, 178], [1736, 234], [-3564, -132], [2222, 96], [1302, 234], [-1776, -114], [3694, 126], [-2052, -244], [-4350, 0], [4536, -94], [7410, 1], [-376, 496], [-2606, 279], [-1926, 332], [-596, 174], [6150, 24], [8036, 24], [2784, 473], [5974, -26], [4194, -41], [716, -580], [-1970, -204], [-3684, 16], [3014, 534], [-3150, -160], [1032, 304], [7312, -131], [1262, 258], [-2996, -248], [10992, -14], [438, -420], [-2054, 381], [-2418, 212], [2426, -568], [1236, -684], [2318, 110], [-1080, -895], [-1630, 432], [3112, 435], [2172, 312], [5406, 384], [-10870, 336], [6990, -390], [-6712, -602], [-2844, 524], [-2424, 367], [-6728, 233], [-852, -900], [14068, 162], [-7680, 457], [6318, -236], [4998, 516], [4988, 1218], [-14726, 21], [-1820, -494], [3630, 358], [-9228, 106], [-7590, -664], [4684, 172], [12098, 488], [1164, -756], [-10062, 532], [-6168, 76], [-1226, -108], [8320, 305], [-514, -328], [-1386, 1134], [17884, -432], [5840, 850], [-11970, -904], [-17756, 804], [8760, -26], [15618, -807], [-20076, -10], [11438, 158], [2878, 274], [474, 1207], [21852, -32], [-23900, -450], [-4530, 30], [-12158, 139], [7320, 883], [2816, -996], [-3658, 1338], [-8620, -858], [-9744, 1489], [13504, -53], [-4450, -896], [11898, 1776], [770, 1404], [-8154, 1028], [-28748, 118], [39390, 318], [-24474, -776], [-14756, -1488], [10554, -1117], [16360, 101], [-20724, 782], [26966, -572], [-5520, 71], [-4190, -1645], [-29802, -412], [34412, 592], [-384, -1600], [-8966, -981], [17694, -426], [-7228, 1530], [-5856, -477], [3436, -2278], [-44808, 63], [-5056, 2880], [-19938, -1132], [-3766, 2432], [-13662, -761], [1092, 110], [-9846, -1784], [17552, -918], [-21204, 818], [-5406, -2352], [9408, -205], [27760, -1353], [49810, -315], [18958, -302], [24042, 211], [-27492, -1770], [-3514, -2070], [5568, -2445], [1898, 1840], [26120, -1948], [42054, -38], [-29060, -576], [15834, -1621], [176, 2562], [-2698, 360], [19496, -2010], [-20028, 1990], [14530, 1065], [15738, -596], [-43872, -1226], [38022, 1946], [49690, -823], [116, -228], [-17402, 458], [28440, -127], [-33602, -200], [-14436, -1036], [12740, 1006], [16638, -1268], [-19212, -1268], [9462, -4250], [46510, 100], [16090, -3003], [-49554, -338], [-27768, 1840], [18690, 3239], [-23936, 465], [-10594, -1384], [39074, -448], [7164, 346], [-20178, -2428], [-3208, 1006], [-1548, 2956], [-30090, -950], [-10900, 1504], [15982, 1586], [-19290, -1480], [-9128, 5391], [21660, -912], [26160, -2273], [454, 5594], [-42568, 1038], [55650, -360], [-55632, -685], [59742, 358], [37838, 2000], [-20216, 369], [25122, -2056], [38128, -1467], [-21516, -3340], [9230, -4874], [7110, -3186], [-1896, 477], [57200, 1252], [-8100, -3116], [2150, 4166], [30484, 3588], [-19480, -3510], [-54390, -912], [-79780, 1802], [11184, -1828], [-87890, -522], [8010, 2955], [49260, -1002], [28296, -1400], [32484, 1536], [-1012, 988], [-86992, 1512], [38966, -3960], [-4962, -5590], [57456, 157], [35248, 1009], [16548, 996], [58604, -60], [59904, -328], [-29002, -2376], [67566, -830], [88872, -750], [-28634, 856], [46302, -3320], [63588, -1764], [-91598, 1841], [44476, 3192], [-38742, -5801], [40106, 1584], [28288, 1629], [62244, -2722], [27202, -1907], [-47222, -3797], [-35202, -716], [8324, -1122], [18414, 750], [42618, 1112], [15100, 3912], [-1386, -7384], [-38726, 4011], [-7132, -2036], [-17398, 1896], [53240, 1106], [10466, 3864], [-77152, -2006], [-32676, -534], [12710, -4482], [55850, 720], [5700, 1894], [-21552, -822], [-94376, 2889], [-2688, -957], [14558, -4018], [-9476, -7086], [-73968, -1744], [-82994, -1734], [15690, -2329], [-124336, 1400], [2226, 3992], [19554, 4369], [-540, -356], [56778, 6104], [24060, -2326], [-31238, -1035], [38130, 2257], [-8392, 2766], [23478, -4154], [-13668, 7552], [-49996, -1840], [-30070, 6588], [942, 228], [59248, -1809], [-38268, -5756], [34060, 2980], [48958, 2862], [-79812, -274], [53962, -4243], [81480, 916], [-20962, -1064], [-90120, 1321], [86970, -1303], [134142, 64], [-2236, -1158], [98376, 4240], [52490, 1608], [-99180, -3924], [-83448, 3017], [49342, -420], [16194, 4920], [16660, -9020], [46774, 1566], [-66450, 3996], [110312, 3246], [-78522, 1716], [-117838, 3132], [5380, -4710], [30364, 6282], [4440, 12375], [-69030, -3709], [71626, 131], [46194, -5085], [120640, -501], [36900, -1766], [-38852, 996], [-40836, 166], [-44890, -4642], [-120210, 4934], [4102, 2964], [42200, 3678], [-73138, 594], [-92070, 5416], [-27510, -7049], [-14272, -4134], [18318, 2066], [-77964, 520], [12418, -7395], [100940, 5310], [90936, -8555], [60090, 6331], [24308, -600], [17062, -4594], [81054, 792], [-80528, 1413], [-73810, -3176], [19050, 7628], [-2640, 11608], [34284, 11120], [63726, 4824], [131592, 2970], [-51182, 6849], [-20778, 5858], [93936, -5306], [-10308, 4464], [-33004, -342], [-105718, 6936], [594, 309], [-5968, -3300], [106774, -5258], [13380, 11094], [96460, 9646], [103224, -7295], [158454, -3086], [-85696, 552], [-155270, -1673], [97620, -9740], [71496, -97], [-24094, 4672], [-71886, -3768], [-88822, 4384], [-1326, 185], [29274, -13840], [-7592, -8137], [70542, 7914], [-46124, -3380], [-131712, -3253], [39272, -9420], [45278, 2572], [9724, 12588], [36720, 4424], [-46850, -7368], [65562, 9271], [-148404, -6194], [58868, 13392], [-82872, 234], [-109034, 9682], [-13192, -7720], [-114726, -6536], [197260, -1698], [29862, 2622], [-157526, -5727], [-166158, -6896], [-62200, -11130], [136086, -9220], [-120666, -1336], [-87224, 5577], [12902, -3652], [142338, -2944], [-70780, 2652], [187044, -1476], [-127826, -4368], [-99930, 7324], [134604, 4178], [-88220, -7330], [-56854, -12920], [-69678, 2075], [-156984, 541], [82408, 10513], [100686, 1532], [-79512, 3298], [-82818, 8262], [-136526, 6219], [50856, -8593], [-28116, -13424], [-106970, 6654], [126462, -276], [15636, 6594], [-47130, 2242], [14828, 2346], [35712, 6059]]
